@@ -15,11 +15,11 @@
 use crate::compiler::args::*;
 use crate::compiler::c::{ArtifactDescriptor, CCompilerImpl, CCompilerKind, ParsedArguments};
 use crate::compiler::{
-    clang, CCompileCommand, Cacheable, ColorMode, CompileCommand, CompilerArguments, Language,
-    SingleCompileCommand,
+    CCompileCommand, Cacheable, ColorMode, CompileCommand, CompilerArguments, Language,
+    SingleCompileCommand, clang,
 };
 use crate::mock_command::{CommandCreatorSync, RunCommand};
-use crate::util::{run_input_output, OsStrExt};
+use crate::util::{OsStrExt, run_input_output};
 use crate::{counted_array, dist};
 use async_trait::async_trait;
 use fs::File;
@@ -172,12 +172,14 @@ use self::ArgData::*;
 
 const ARCH_FLAG: &str = "-arch";
 
-// Mostly taken from https://github.com/ccache/ccache/blob/master/src/compopt.cpp#L52-L172
+// Mostly taken from https://github.com/ccache/ccache/blob/master/src/ccache/compopt.cpp#L52-L183
 counted_array!(pub static ARGS: [ArgInfo<ArgData>; _] = [
     flag!("-", TooHardFlag),
     flag!("--coverage", Coverage),
     take_arg!("--param", OsString, Separated, PassThrough),
     flag!("--save-temps", TooHardFlag),
+    flag!("--save-temps=cwd", TooHardFlag),
+    flag!("--save-temps=obj", TooHardFlag),
     take_arg!("--serialize-diagnostics", PathBuf, Separated, SerializeDiagnostics),
     take_arg!("--sysroot", PathBuf, Separated, PassThroughPath),
     take_arg!("-A", OsString, Separated, PassThrough),
@@ -200,7 +202,7 @@ counted_array!(pub static ARGS: [ArgInfo<ArgData>; _] = [
     take_arg!("-U", OsString, CanBeSeparated, PassThrough),
     take_arg!("-V", OsString, Separated, PassThrough),
     flag!("-Werror=pedantic", PedanticFlag),
-    take_arg!("-Wp", OsString, Concatenated(','), PreprocessorArgument),
+    take_arg!("-Wp", OsString, Concatenated(b','), PreprocessorArgument),
     flag!("-Wpedantic", PedanticFlag),
     take_arg!("-Xassembler", OsString, Separated, PassThrough),
     take_arg!("-Xlinker", OsString, Separated, PassThrough),
@@ -209,7 +211,7 @@ counted_array!(pub static ARGS: [ArgInfo<ArgData>; _] = [
     take_arg!("-aux-info", OsString, Separated, PassThrough),
     take_arg!("-b", OsString, Separated, PassThrough),
     flag!("-c", DoCompilation),
-    take_arg!("-fdiagnostics-color", OsString, Concatenated('='), DiagnosticsColor),
+    take_arg!("-fdiagnostics-color", OsString, Concatenated(b'='), DiagnosticsColor),
     flag!("-fno-diagnostics-color", NoDiagnosticsColorFlag),
     flag!("-fno-profile-generate", TooHardFlag),
     flag!("-fno-profile-use", TooHardFlag),
@@ -242,10 +244,13 @@ counted_array!(pub static ARGS: [ArgInfo<ArgData>; _] = [
     take_arg!("-o", PathBuf, CanBeSeparated, Output),
     flag!("-pedantic", PedanticFlag),
     flag!("-pedantic-errors", PedanticFlag),
+    flag!("-pipe", UnhashedFlag),
     flag!("-remap", PreprocessorArgumentFlag),
     flag!("-save-temps", TooHardFlag),
-    take_arg!("-std", OsString, Concatenated('='), Standard),
-    take_arg!("-stdlib", OsString, Concatenated('='), PreprocessorArgument),
+    flag!("-save-temps=cwd", TooHardFlag),
+    flag!("-save-temps=obj", TooHardFlag),
+    take_arg!("-std", OsString, Concatenated(b'='), Standard),
+    take_arg!("-stdlib", OsString, Concatenated(b'='), PreprocessorArgument),
     flag!("-trigraphs", PreprocessorArgumentFlag),
     take_arg!("-u", OsString, CanBeSeparated, PassThrough),
     take_arg!("-x", OsString, CanBeSeparated, Language),
@@ -372,7 +377,6 @@ where
             }
             Some(Output(p)) => output_arg = Some(p.clone()),
             Some(NeedDepTarget) => {
-                too_hard_for_preprocessor_cache_mode = Some(arg.to_os_string());
                 need_explicit_dep_target = true;
                 if let DepArgumentRequirePath::NotNeeded = need_explicit_dep_argument_path {
                     need_explicit_dep_argument_path = DepArgumentRequirePath::Missing;
@@ -400,6 +404,8 @@ where
             | Some(Unhashed(_)) => {}
             Some(Language(lang)) => {
                 language = match lang.to_string_lossy().as_ref() {
+                    "assembler" => Some(Language::Assembler),
+                    "assembler-with-cpp" => Some(Language::AssemblerToPreprocess),
                     "c" => Some(Language::C),
                     "c-header" => Some(Language::CHeader),
                     "c++" => Some(Language::Cxx),
@@ -422,7 +428,7 @@ where
                         )
                     }
                     _ => {}
-                };
+                }
                 seen_arch = Some(arch.clone());
             }
             Some(XClang(s)) => xclangs.push(s.clone()),
@@ -515,9 +521,10 @@ where
             | Some(Output(_))
             | Some(TooHardFlag)
             | Some(XClang(_))
-            | Some(TooHard(_)) => cannot_cache!(arg
-                .flag_str()
-                .unwrap_or("Can't handle complex arguments through clang",)),
+            | Some(TooHard(_)) => cannot_cache!(
+                arg.flag_str()
+                    .unwrap_or("Can't handle complex arguments through clang",)
+            ),
             None => match arg {
                 Argument::Raw(_) if follows_plugin_arg => &mut common_args,
                 Argument::Raw(flag) => cannot_cache!(
@@ -566,7 +573,7 @@ where
         };
         for arg in arg.normalize(norm).iter_os_strings() {
             args.push("-Xclang".into());
-            args.push(arg)
+            args.push(arg);
         }
     }
 
@@ -695,21 +702,7 @@ where
 }
 
 pub fn language_to_gcc_arg(lang: Language) -> Option<&'static str> {
-    match lang {
-        Language::C => Some("c"),
-        Language::CHeader => Some("c-header"),
-        Language::Cxx => Some("c++"),
-        Language::CxxHeader => Some("c++-header"),
-        Language::ObjectiveC => Some("objective-c"),
-        Language::ObjectiveCxx => Some("objective-c++"),
-        Language::ObjectiveCxxHeader => Some("objective-c++-header"),
-        Language::Cuda => Some("cu"),
-        Language::Ptx => None,
-        Language::Cubin => None,
-        Language::Rust => None, // Let the compiler decide
-        Language::Hip => Some("hip"),
-        Language::GenericHeader => None, // Let the compiler decide
-    }
+    lang.to_gcc_arg()
 }
 
 #[allow(clippy::too_many_arguments)]
@@ -867,7 +860,7 @@ where
     let language = language_to_arg(parsed_args.language);
     let mut arguments: Vec<OsString> = vec![];
     if let Some(lang) = &language {
-        arguments.extend(vec!["-x".into(), lang.into()])
+        arguments.extend(vec!["-x".into(), lang.into()]);
     }
     arguments.extend(vec![
         parsed_args.compilation_flag.clone(),
@@ -926,7 +919,7 @@ where
             let mut arguments: Vec<String> = vec![];
             // Language needs to be before input
             if let Some(lang) = &language {
-                arguments.extend(vec!["-x".into(), lang.into()])
+                arguments.extend(vec!["-x".into(), lang.into()]);
             }
             arguments.extend(vec![
                 parsed_args.compilation_flag.clone().into_string().ok()?,
@@ -980,15 +973,12 @@ impl<'a> ExpandIncludeFile<'a> {
     }
 }
 
-impl<'a> Iterator for ExpandIncludeFile<'a> {
+impl Iterator for ExpandIncludeFile<'_> {
     type Item = OsString;
 
     fn next(&mut self) -> Option<OsString> {
         loop {
-            let arg = match self.stack.pop() {
-                Some(arg) => arg,
-                None => return None,
-            };
+            let arg = self.stack.pop()?;
             let file = match arg.split_prefix("@") {
                 Some(arg) => self.cwd.join(arg),
                 None => return Some(arg),
@@ -1479,6 +1469,25 @@ mod test {
         assert!(!msvc_show_includes);
     }
 
+    #[test]
+    fn test_parse_arguments_unhashed() {
+        let unhashed_flags = stringvec!["-pipe"];
+
+        for flag in unhashed_flags {
+            let args = stringvec!["-c", "foo.c", "-o", "foo.o", flag];
+
+            let a = match parse_arguments_(args, false) {
+                CompilerArguments::Ok(args) => args,
+                o => panic!("Got unexpected parse result: {:?} for flag: {:?}", o, flag),
+            };
+
+            assert_eq!(Language::C, a.language);
+            assert!(a.preprocessor_args.is_empty());
+            assert!(a.common_args.is_empty());
+            assert_eq!(ovec![flag], a.unhashed_args,);
+        }
+    }
+
     #[test]
     fn test_parse_arguments_double_dash() {
         let args = stringvec!["-c", "-o", "foo.o", "--", "foo.c"];
@@ -1538,10 +1547,36 @@ mod test {
         );
     }
 
+    #[test]
+    fn test_parse_arguments_too_hard() {
+        let too_hard_flags = stringvec![
+            "-save-temps",
+            "-save-temps=cwd",
+            "-save-temps=obj",
+            "--save-temps",
+            "--save-temps=cwd",
+            "--save-temps=obj"
+        ];
+
+        for flag in too_hard_flags {
+            let args = stringvec!["-c", "foo.c", "-o", "foo.o", flag];
+
+            match parse_arguments_(args, false) {
+                CompilerArguments::CannotCache(reason, None) => {
+                    assert_eq!(reason, &flag);
+                }
+                o => {
+                    panic!("Got unexpected parse result: {:?} for flag: {:?}", o, flag);
+                }
+            }
+        }
+    }
+
     #[test]
     fn test_parse_arguments_explicit_dep_target() {
-        let args =
-            stringvec!["-c", "foo.c", "-MT", "depfile", "-fabc", "-MF", "foo.o.d", "-o", "foo.o"];
+        let args = stringvec![
+            "-c", "foo.c", "-MT", "depfile", "-fabc", "-MF", "foo.o.d", "-o", "foo.o"
+        ];
         let ParsedArguments {
             input,
             language,
@@ -1876,7 +1911,9 @@ mod test {
 
     #[test]
     fn test_parse_arguments_dep_target_needed() {
-        let args = stringvec!["-c", "foo.c", "-fabc", "-MF", "foo.o.d", "-o", "foo.o", "-MD"];
+        let args = stringvec![
+            "-c", "foo.c", "-fabc", "-MF", "foo.o.d", "-o", "foo.o", "-MD"
+        ];
         let ParsedArguments {
             input,
             language,
@@ -2078,7 +2115,9 @@ mod test {
 
         with_var("SCCACHE_CACHE_MULTIARCH", Some("1"), || {
             match parse_arguments_(
-                stringvec!["-arch", "arm64", "-arch", "arm64", "-o", "foo.o", "-c", "foo.cpp"],
+                stringvec![
+                    "-arch", "arm64", "-arch", "arm64", "-o", "foo.o", "-c", "foo.cpp"
+                ],
                 false,
             ) {
                 CompilerArguments::Ok(_) => {}
