@@ -17,7 +17,8 @@ use blake3::Hasher as blake3_Hasher;
 use byteorder::{BigEndian, ByteOrder};
 use fs::File;
 use fs_err as fs;
-use object::{macho, read::archive::ArchiveFile, read::macho::FatArch};
+use object::read::archive::ArchiveFile;
+use object::read::macho::{FatArch, MachOFatFile32, MachOFatFile64};
 use serde::{Deserialize, Serialize};
 use std::cell::Cell;
 use std::ffi::{OsStr, OsString};
@@ -199,7 +200,7 @@ impl TimeMacroFinder {
                 if !self.previous_small_read.is_empty() {
                     // In a rare pathological case where all reads are small,
                     // this will grow up to the length of the file.
-                    // It it *very* unlikely and of minor performance
+                    // It is *very* unlikely and of minor performance
                     // importance compared to just getting many small reads.
                     self.previous_small_read.extend(visit);
                 } else {
@@ -276,10 +277,10 @@ impl TimeMacroFinder {
         }
         if memchr::memmem::find(buffer, b"__TIME__").is_some() {
             self.found_time.set(true);
-        };
+        }
         if memchr::memmem::find(buffer, b"__DATE__").is_some() {
             self.found_date.set(true);
-        };
+        }
     }
 
     pub fn found_time_macros(&self) -> bool {
@@ -356,20 +357,19 @@ pub async fn hash_all_archives(
             let archive_mmap =
                 unsafe { memmap2::MmapOptions::new().map_copy_read_only(&archive_file)? };
 
-            match macho::FatHeader::parse(&*archive_mmap) {
-                Ok(h) if h.magic.get(object::endian::BigEndian) == macho::FAT_MAGIC => {
-                    for arch in macho::FatHeader::parse_arch32(&*archive_mmap)? {
-                        hash_regular_archive(&mut m, arch.data(&*archive_mmap)?)?;
-                    }
+            if let Ok(fat) = MachOFatFile32::parse(&*archive_mmap) {
+                for arch in fat.arches() {
+                    hash_regular_archive(&mut m, arch.data(&*archive_mmap)?)?;
                 }
-                Ok(h) if h.magic.get(object::endian::BigEndian) == macho::FAT_MAGIC_64 => {
-                    for arch in macho::FatHeader::parse_arch64(&*archive_mmap)? {
-                        hash_regular_archive(&mut m, arch.data(&*archive_mmap)?)?;
-                    }
+            } else if let Ok(fat) = MachOFatFile64::parse(&*archive_mmap) {
+                for arch in fat.arches() {
+                    hash_regular_archive(&mut m, arch.data(&*archive_mmap)?)?;
                 }
+            } else {
                 // Not a FatHeader at all, regular archive.
-                _ => hash_regular_archive(&mut m, &archive_mmap)?,
+                hash_regular_archive(&mut m, &archive_mmap)?;
             }
+
             Ok(m.finish())
         })
     });
@@ -646,7 +646,7 @@ pub fn decode_path(bytes: &[u8]) -> std::io::Result<PathBuf> {
 
 #[cfg(windows)]
 pub fn wide_char_to_multi_byte(wide_char_str: &[u16]) -> std::io::Result<Vec<u8>> {
-    use windows_sys::Win32::Globalization::{WideCharToMultiByte, CP_OEMCP};
+    use windows_sys::Win32::Globalization::{CP_OEMCP, WideCharToMultiByte};
 
     let codepage = CP_OEMCP;
     let flags = 0;
@@ -780,7 +780,7 @@ impl From<std::time::SystemTime> for Timestamp {
                     nanoseconds = NSEC_PER_SEC - negative_nanos;
                 }
             }
-        };
+        }
         Self {
             seconds,
             nanoseconds,
@@ -829,9 +829,9 @@ pub struct HashToDigest<'a> {
     pub digest: &'a mut Digest,
 }
 
-impl<'a> Hasher for HashToDigest<'a> {
+impl Hasher for HashToDigest<'_> {
     fn write(&mut self, bytes: &[u8]) {
-        self.digest.update(bytes)
+        self.digest.update(bytes);
     }
 
     fn finish(&self) -> u64 {
@@ -843,7 +843,7 @@ impl<'a> Hasher for HashToDigest<'a> {
 #[cfg(not(windows))]
 pub fn daemonize() -> Result<()> {
     use crate::jobserver::discard_inherited_jobserver;
-    use daemonize::Daemonize;
+    use daemonix::Daemonize;
     use std::env;
     use std::mem;
 
@@ -858,9 +858,9 @@ pub fn daemonize() -> Result<()> {
         discard_inherited_jobserver();
     }
 
-    static mut PREV_SIGSEGV: *mut libc::sigaction = 0 as *mut _;
-    static mut PREV_SIGBUS: *mut libc::sigaction = 0 as *mut _;
-    static mut PREV_SIGILL: *mut libc::sigaction = 0 as *mut _;
+    static mut PREV_SIGSEGV: *mut libc::sigaction = std::ptr::null_mut();
+    static mut PREV_SIGBUS: *mut libc::sigaction = std::ptr::null_mut();
+    static mut PREV_SIGILL: *mut libc::sigaction = std::ptr::null_mut();
 
     // We don't have a parent process any more once we've reached this point,
     // which means that no one's probably listening for our exit status.
@@ -883,7 +883,7 @@ pub fn daemonize() -> Result<()> {
         PREV_SIGBUS = Box::into_raw(Box::new(mem::zeroed::<libc::sigaction>()));
         PREV_SIGILL = Box::into_raw(Box::new(mem::zeroed::<libc::sigaction>()));
         let mut new: libc::sigaction = mem::zeroed();
-        new.sa_sigaction = handler as usize;
+        new.sa_sigaction = (handler as *const libc::c_void).expose_provenance();
         new.sa_flags = libc::SA_SIGINFO | libc::SA_RESTART;
         libc::sigaction(libc::SIGSEGV, &new, &mut *PREV_SIGSEGV);
         libc::sigaction(libc::SIGBUS, &new, &mut *PREV_SIGBUS);
@@ -992,7 +992,7 @@ pub fn ascii_unescape_default(s: &[u8]) -> std::io::Result<Vec<u8>> {
                             "incomplete hex escape",
                         ));
                     }
-                    let v = unhex(s[offset])? << 4 | unhex(s[offset + 1])?;
+                    let v = (unhex(s[offset])? << 4) | unhex(s[offset + 1])?;
                     out.push(v);
                     offset += 1;
                 }
@@ -1011,6 +1011,10 @@ pub fn ascii_unescape_default(s: &[u8]) -> std::io::Result<Vec<u8>> {
     Ok(out)
 }
 
+pub fn num_cpus() -> usize {
+    std::thread::available_parallelism().map_or(1, std::num::NonZeroUsize::get)
+}
+
 #[cfg(test)]
 mod tests {
     use super::{OsStrExt, TimeMacroFinder};
