@@ -1,9 +1,11 @@
 use fs_err as fs;
+use log::trace;
 #[cfg(any(feature = "dist-client", feature = "dist-server"))]
 use sccache::config::HTTPUrl;
 use sccache::dist::{self, SchedulerStatusResult, ServerId};
 use sccache::server::ServerInfo;
 use std::env;
+use std::fs::remove_dir_all;
 use std::io::Write;
 use std::net::{self, IpAddr, SocketAddr};
 use std::path::{Path, PathBuf};
@@ -30,6 +32,7 @@ const DIST_IMAGE: &str = "sccache_dist_test_image";
 const DIST_DOCKERFILE: &str = include_str!("Dockerfile.sccache-dist");
 const DIST_IMAGE_BWRAP_PATH: &str = "/usr/bin/bwrap";
 const MAX_STARTUP_WAIT: Duration = Duration::from_secs(5);
+const DIST_CACHE_RELPATH: &str = "client-dist-cache";
 
 const DIST_SERVER_TOKEN: &str = "THIS IS THE TEST TOKEN";
 
@@ -46,7 +49,8 @@ pub fn start_local_daemon(cfg_path: &Path, cached_cfg_path: &Path) {
     if !sccache_command()
         .arg("--start-server")
         // Uncomment following lines to debug locally.
-        // .env("SCCACHE_LOG", "debug")
+        // .env("SCCACHE_LOG", "sccache=trace")
+        // .env("RUST_LOG_STYLE", "never")
         // .env(
         //     "SCCACHE_ERROR_LOG",
         //     env::temp_dir().join("sccache_local_daemon.txt"),
@@ -68,7 +72,14 @@ pub fn stop_local_daemon() -> bool {
         .stdout(Stdio::null())
         .stderr(Stdio::null())
         .status()
-        .map_or(false, |status| status.success())
+        .is_ok_and(|status| status.success())
+}
+
+pub fn clear_cache_local_daemon(tmpdir: &Path) -> bool {
+    trace!("clear local cache daemon");
+    let client_build_dir = tmpdir.join(DIST_CACHE_RELPATH);
+    assert!(client_build_dir.exists());
+    remove_dir_all(client_build_dir).is_ok()
 }
 
 pub fn get_stats<F: 'static + Fn(ServerInfo)>(f: F) {
@@ -109,11 +120,17 @@ pub fn write_source(path: &Path, filename: &str, contents: &str) {
     f.write_all(contents.as_bytes()).unwrap();
 }
 
+pub fn init_cargo(path: &Path, cargo_name: &str) -> PathBuf {
+    let cargo_path = path.join(cargo_name);
+    let source_path = "src";
+    fs::create_dir_all(cargo_path.join(source_path)).unwrap();
+    cargo_path
+}
+
 // Prune any environment variables that could adversely affect test execution.
-pub fn sccache_command() -> Command {
+pub fn prune_command(mut cmd: Command) -> Command {
     use sccache::util::OsStrExt;
 
-    let mut cmd = Command::new(assert_cmd::cargo::cargo_bin("sccache"));
     for (var, _) in env::vars_os() {
         if var.starts_with("SCCACHE_") {
             cmd.env_remove(var);
@@ -122,9 +139,17 @@ pub fn sccache_command() -> Command {
     cmd
 }
 
+pub fn sccache_command() -> Command {
+    prune_command(Command::new(env!("CARGO_BIN_EXE_sccache")))
+}
+
+pub fn cargo_command() -> Command {
+    prune_command(Command::new("cargo"))
+}
+
 #[cfg(feature = "dist-server")]
 pub fn sccache_dist_path() -> PathBuf {
-    assert_cmd::cargo::cargo_bin("sccache-dist")
+    env!("CARGO_BIN_EXE_sccache-dist").into()
 }
 
 pub fn sccache_client_cfg(
@@ -132,7 +157,7 @@ pub fn sccache_client_cfg(
     preprocessor_cache_mode: bool,
 ) -> sccache::config::FileConfig {
     let cache_relpath = "client-cache";
-    let dist_cache_relpath = "client-dist-cache";
+    let dist_cache_relpath = DIST_CACHE_RELPATH;
     fs::create_dir(tmpdir.join(cache_relpath)).unwrap();
     fs::create_dir(tmpdir.join(dist_cache_relpath)).unwrap();
 
@@ -195,6 +220,7 @@ fn sccache_server_cfg(
         },
         cache_dir: Path::new(CONFIGS_CONTAINER_PATH).join(relpath),
         public_addr: SocketAddr::new(server_ip, SERVER_PORT),
+        bind_address: Some(SocketAddr::from(([0, 0, 0, 0], SERVER_PORT))),
         scheduler_url,
         scheduler_auth: sccache::config::server::SchedulerAuth::Token {
             token: DIST_SERVER_TOKEN.to_owned(),
@@ -211,6 +237,7 @@ fn create_server_token(server_id: ServerId, auth_token: &str) -> String {
 }
 
 #[cfg(feature = "dist-server")]
+#[allow(dead_code)]
 pub enum ServerHandle {
     Container { cid: String, url: HTTPUrl },
     Process { pid: Pid, url: HTTPUrl },
@@ -286,10 +313,10 @@ impl DistSystem {
                 "--network",
                 "host",
                 "-v",
-                &format!("{}:/sccache-dist", self.sccache_dist.to_str().unwrap()),
+                &format!("{}:/sccache-dist:z", self.sccache_dist.to_str().unwrap()),
                 "-v",
                 &format!(
-                    "{}:{}",
+                    "{}:{}:z",
                     self.tmpdir.to_str().unwrap(),
                     CONFIGS_CONTAINER_PATH
                 ),
@@ -354,10 +381,10 @@ impl DistSystem {
                 "--network",
                 "host",
                 "-v",
-                &format!("{}:/sccache-dist", self.sccache_dist.to_str().unwrap()),
+                &format!("{}:/sccache-dist:z", self.sccache_dist.to_str().unwrap()),
                 "-v",
                 &format!(
-                    "{}:{}",
+                    "{}:{}:z",
                     self.tmpdir.to_str().unwrap(),
                     CONFIGS_CONTAINER_PATH
                 ),
@@ -409,16 +436,23 @@ impl DistSystem {
             listener.local_addr().unwrap()
         };
         let token = create_server_token(ServerId::new(server_addr), DIST_SERVER_TOKEN);
-        let server =
-            dist::http::Server::new(server_addr, self.scheduler_url().to_url(), token, handler)
-                .unwrap();
+        let server = dist::http::Server::new(
+            server_addr,
+            Some(SocketAddr::from(([0, 0, 0, 0], server_addr.port()))),
+            self.scheduler_url().to_url(),
+            token,
+            handler,
+        )
+        .unwrap();
         let pid = match unsafe { nix::unistd::fork() }.unwrap() {
             ForkResult::Parent { child } => {
                 self.server_pids.push(child);
                 child
             }
             ForkResult::Child => {
-                env::set_var("SCCACHE_LOG", "sccache=trace");
+                unsafe {
+                    env::set_var("SCCACHE_LOG", "sccache=trace");
+                }
                 env_logger::try_init().unwrap();
                 server.start().unwrap();
                 unreachable!();
@@ -448,6 +482,30 @@ impl DistSystem {
         }
         self.wait_server_ready(handle)
     }
+    pub fn count_toolchains_on_server(&mut self, handle: &ServerHandle) -> usize {
+        match handle {
+            ServerHandle::Container { cid, url: _ } => {
+                let output = Command::new("docker")
+                    .args([
+                        "exec",
+                        "-t",
+                        cid,
+                        "ls",
+                        "-1",
+                        &format!("{BUILD_DIR_CONTAINER_PATH}/toolchains"),
+                    ])
+                    .output()
+                    .unwrap();
+                check_output(&output);
+                println!("{output:?}");
+                let str_output = String::from_utf8_lossy(&output.stdout);
+                str_output.lines().filter(|&part| !part.is_empty()).count()
+            }
+            ServerHandle::Process { pid: _, url: _ } => {
+                panic!("restart not yet implemented for pids")
+            }
+        }
+    }
 
     pub fn wait_server_ready(&mut self, handle: &ServerHandle) {
         let url = match handle {
@@ -518,32 +576,44 @@ impl Drop for DistSystem {
         let mut exits = vec![];
 
         if let Some(scheduler_name) = self.scheduler_name.as_ref() {
-            droperr!(Command::new("docker")
-                .args(["logs", scheduler_name])
-                .output()
-                .map(|o| logs.push((scheduler_name, o))));
-            droperr!(Command::new("docker")
-                .args(["kill", scheduler_name])
-                .output()
-                .map(|o| outputs.push((scheduler_name, o))));
-            droperr!(Command::new("docker")
-                .args(["rm", "-f", scheduler_name])
-                .output()
-                .map(|o| outputs.push((scheduler_name, o))));
+            droperr!(
+                Command::new("docker")
+                    .args(["logs", scheduler_name])
+                    .output()
+                    .map(|o| logs.push((scheduler_name, o)))
+            );
+            droperr!(
+                Command::new("docker")
+                    .args(["kill", scheduler_name])
+                    .output()
+                    .map(|o| outputs.push((scheduler_name, o)))
+            );
+            droperr!(
+                Command::new("docker")
+                    .args(["rm", "-f", scheduler_name])
+                    .output()
+                    .map(|o| outputs.push((scheduler_name, o)))
+            );
         }
         for server_name in self.server_names.iter() {
-            droperr!(Command::new("docker")
-                .args(["logs", server_name])
-                .output()
-                .map(|o| logs.push((server_name, o))));
-            droperr!(Command::new("docker")
-                .args(["kill", server_name])
-                .output()
-                .map(|o| outputs.push((server_name, o))));
-            droperr!(Command::new("docker")
-                .args(["rm", "-f", server_name])
-                .output()
-                .map(|o| outputs.push((server_name, o))));
+            droperr!(
+                Command::new("docker")
+                    .args(["logs", server_name])
+                    .output()
+                    .map(|o| logs.push((server_name, o)))
+            );
+            droperr!(
+                Command::new("docker")
+                    .args(["kill", server_name])
+                    .output()
+                    .map(|o| outputs.push((server_name, o)))
+            );
+            droperr!(
+                Command::new("docker")
+                    .args(["rm", "-f", server_name])
+                    .output()
+                    .map(|o| outputs.push((server_name, o)))
+            );
         }
         for &pid in self.server_pids.iter() {
             droperr!(nix::sys::signal::kill(pid, Signal::SIGINT));
@@ -560,14 +630,16 @@ impl Drop for DistSystem {
             if killagain {
                 eprintln!("SIGINT didn't kill process, trying SIGKILL");
                 droperr!(nix::sys::signal::kill(pid, Signal::SIGKILL));
-                droperr!(nix::sys::wait::waitpid(pid, Some(WaitPidFlag::WNOHANG))
-                    .map_err(|e| e.to_string())
-                    .and_then(|ws| if ws == WaitStatus::StillAlive {
-                        Err("process alive after sigkill".to_owned())
-                    } else {
-                        exits.push(ws);
-                        Ok(())
-                    }));
+                droperr!(
+                    nix::sys::wait::waitpid(pid, Some(WaitPidFlag::WNOHANG))
+                        .map_err(|e| e.to_string())
+                        .and_then(|ws| if ws == WaitStatus::StillAlive {
+                            Err("process alive after sigkill".to_owned())
+                        } else {
+                            exits.push(ws);
+                            Ok(())
+                        })
+                );
             }
         }
 
@@ -626,8 +698,12 @@ fn make_container_name(tag: &str) -> String {
 
 fn check_output(output: &Output) {
     if !output.status.success() {
-        println!("{}\n\n[BEGIN STDOUT]\n===========\n{}\n===========\n[FIN STDOUT]\n\n[BEGIN STDERR]\n===========\n{}\n===========\n[FIN STDERR]\n\n",
-            output.status, String::from_utf8_lossy(&output.stdout), String::from_utf8_lossy(&output.stderr));
+        println!(
+            "{}\n\n[BEGIN STDOUT]\n===========\n{}\n===========\n[FIN STDOUT]\n\n[BEGIN STDERR]\n===========\n{}\n===========\n[FIN STDERR]\n\n",
+            output.status,
+            String::from_utf8_lossy(&output.stdout),
+            String::from_utf8_lossy(&output.stderr)
+        );
         panic!()
     }
 }
